// Stazeni GPS souradnic pres I2C z pridavneho modulu s ATmega328 a s GPS modulem
//================================================

// priklad exportnich dat v registrech:
// 0 ---   8       prumer 10 poslednich LAT
// 1 ---  79
// 2 --- 192
// 3 --- 172

// 4 ---  11       prumer 10 poslednich LON
// 5 --- 149
// 6 --- 204
// 7 --- 147

// 8 --- 100       OK cnt

// 9 ---  21       YY
//10 ---   1       MM
//11 ---  25       DD
//12 ---  11       HH
//13 ---  49       NN
//14 ---  22       SS

//15 ---   1       prumer 10 poslednich ALT
//16 --- 241

//17 ---   4       Pocet satelitu
//18 ---  23       10x HDoP
//19 ---   1       pole klouzaku zaplneno, muzou se stahovat data


//----------------------------------------------
// stazeni dat z GPS rozsirujici desky
void gps(boolean povol_SAVE_RTC)
  {
#ifdef ukladat_GPS 

    if (digitalRead(pin_EXT_ZAP) == LOW)                                      // modul pripojeny
      {
         
       // tady nacitat data z I2C adresy 0x17 (modul GPS) do promennych pole_GPS_I2C[] 
        for (i = 0 ; i < 20 ; i++)
          {
            pole_GPS_I2C[i] = 0;
          }
    
        i=0;
        Wire.requestFrom(I2C_ADDR_GPS, 20);
        delay(100);
        while (Wire.available())
          {
            pole_GPS_I2C[i] = Wire.read();
            i++;
          }
    
        
        boolean problem = false;
        if (i < 19)                                                           // nebyl prijaty prislusny pocet bajtu - NECO JE SPATNE
          {
            zobraz_text(41);                                                  // "Err-2"
            delay(1500);
            zobraz_text(28);                                                  // "     "
            problem = true;
          }
        else                                                                  // pres I2C dorazil ocekavany pocet bajtu
          {
            byte GPS_OK = pole_GPS_I2C[8];                                    // pocet bezchybnych prijmu v rade
    
    
            if (bitRead(pole_GPS_I2C[19],1) == true and pole_GPS_I2C[9] > 20) // prisel spravny pocet bajtu, je vyzadan zapis casu do RTC a rok se zda byt v poradku
              {
                if (povol_SAVE_RTC == true)                                   // v menu nebo pri testu se RTC neprenastavi, i kdyby to vynucene prepinacem na modulu GPS
                  {
                    GPS_nastav_cas();                               
                  }
              }
    
            
            if (bitRead(pole_GPS_I2C[19],0) == true)                          // klouzak uz je pripraveny
              {
                GPS_lat = (pole_GPS_I2C[0] << 24) | (pole_GPS_I2C[1] << 16) | (pole_GPS_I2C[2] << 8) | (pole_GPS_I2C[3]); 
                GPS_lon = (pole_GPS_I2C[4] << 24) | (pole_GPS_I2C[5] << 16) | (pole_GPS_I2C[6] << 8) | (pole_GPS_I2C[7]); 
                GPS_alt = (pole_GPS_I2C[15] << 8) | (pole_GPS_I2C[16]);           
              }
            else
              {
                problem = true;                                               // neni dostatek dat pro klouzak, nebo je signal casto prerusovany
              }
          }
    
    
        if (problem == true)                                                  // pri problemu se prenastavi hodnoty zamerne mimo rozsah, aby se ve vypisu zobrazily pomlcky
          {
            GPS_lat = 0xFFFFFFFFUL;                                           // hodnoty mimo rozsah se ve vypisu zobrazi jako pomlcky
            GPS_lon = 0xFFFFFFFFUL;
            GPS_alt = 0xFFFF;        
          }
      }
    else                                                                      // modul neni zasunuty
      {
        GPS_lat = 0xFFFFFFFFUL;                                               // hodnoty mimo rozsah se ve vypisu zobrazi jako pomlcky
        GPS_lon = 0xFFFFFFFFUL;
        GPS_alt = 0xFFFF;              
      }
   


#else
    byte GPS_OK = 0;
    GPS_lat = 0xFFFFFFFFUL;
    GPS_lon = 0xFFFFFFFFUL;
    GPS_alt = 0xFFFF;
#endif

#ifdef ukladat_GPS

    if (posledni_i2C_prikaz != 21)
      {
        posledni_i2C_prikaz = 21;
        if (digitalRead(pin_EXT_ZAP) == LOW)                                  // s modulem se komunikuje jen kdyz je pripojeny
          {
            Wire.beginTransmission(0x17);                                     // LED na GPS modulu po skonceni mereni obnovit na hodnoty, ktere byly pred nucenym zhasnutim
            Wire.write(21);
            Wire.endTransmission();
          }
      }
#endif
  }
//----------------------------------------------



//----------------------------------------------
// zobrazeni souradnic na 5-mistnem displeji (vcetne pripadneho scrolovani)
void disp_GPS(unsigned long souradnice, byte smer_osy, boolean scroll)
  { 

    if (souradnice == 0xFFFFFFFFUL)                                           // pri chybe GPS (jeste nejsou k dispozici spravne souradnice, nebo neni vubec pripojeny modul)
      {
        zobraz_text(29);                                                      // na displeji se zobrazi pomlcky bez scrolovani "-----" 
      }
    else                                                                      // souradnice v poradku
      {  
        pom_disp_scroll[0] = 0;                                               // prazdna sedmisegmentovka
        pom_disp_scroll[1] = 0;                                               // prazdna sedmisegmentovka
        pom_disp_scroll[2] = 0;                                               // prazdna sedmisegmentovka
        pom_disp_scroll[3] = 0;                                               // prazdna sedmisegmentovka
        
        if(smer_osy == 2)                                                     // nadmorska vyska
          {
            if (souradnice > 500)                                             // nad morem
              {
                souradnice = souradnice - 500;                                // odecita se 500 metru
                pom_disp_scroll[4] = 0;                                       // prazdna sedmisegmentovka
                
              }
            else                                                              // pod morem
              {
                pom_disp_scroll[4] = 64;                                      // pomlcka na sedmisegmentovce
              }     
          }

        
        
        if(smer_osy == 0)                                                     // zemepisna sirka (LAT)
          {
            if (souradnice > 90000000UL)                                      // severni polokoule
              {
                souradnice = souradnice - 90000000UL;                         // pro sever se odecita 90 stupnu
                pom_disp_scroll[4] = 1;                                       // horni pomlcka na sedmisegmentovce
                
              }
            else
              {
                pom_disp_scroll[4] = 64 + 8;                                  // pro jih se zobrazuje pomlcka a podtrzitko na prvni sedmisegmentovce 
              }     
          }
          
        if (smer_osy == 2)                                                    // zemepisna delka (LON)
          {
            if (souradnice > 180000000UL)                                     // vychodni polokoule
              {
                souradnice = souradnice - 180000000UL;                        // pro vychod se odecita 180 stupnu
                pom_disp_scroll[4] = 2;                                       // pravy (vychodni) apostrof na prvni sedmisegmentovce
                
              }
            else
              {
                pom_disp_scroll[4] = 64 + 32;                                 // pro zapad se zobrazi pomlcka a levy (zapadni) apostrof na sedmisegmentovce
              }     
          }
      
        if (smer_osy != 1)                                                    // pro LAT nebo LON se provadi uvodni scroll
          {
            unsigned long pom_souradnice = souradnice;
    
            unsigned long mocnina = 100000000UL;
            for (byte rad = 5; rad < 14 ; rad ++)
              {
    
                if (pom_souradnice < mocnina)
                  {
                    pom_disp_scroll[rad] = graf_def[0];
                  }
                else
                  {
                    pom_disp_scroll[rad] = graf_def[pom_souradnice / mocnina];
                    pom_souradnice       =  pom_souradnice % mocnina;
                  }
    
                if (rad == 7) pom_disp_scroll[rad] = pom_disp_scroll[rad] + 128;      // desetinna tecka na displej
    
                mocnina = mocnina / 10UL;                                     // prechod na nizsi dekadu
              }
        
            if (scroll == true)                                               // postupne scrollovani cisla pres displej
              {
                for (byte i = 0; i< 10 ; i++)
                  {
                    D_pamet[4] =  pom_disp_scroll[i];
                    D_pamet[3] =  pom_disp_scroll[i+1];
                    D_pamet[2] =  pom_disp_scroll[i+2];
                    D_pamet[1] =  pom_disp_scroll[i+3];
                    D_pamet[0] =  pom_disp_scroll[i+4];
                    aktualizuj_displej();
                    delay(500);
                  }
              }
            else                                                              // bez scrollovani - jen poslednich 5 mist
              {
                D_pamet[4] =  pom_disp_scroll[9];
                D_pamet[3] =  pom_disp_scroll[10];
                D_pamet[2] =  pom_disp_scroll[11];
                D_pamet[1] =  pom_disp_scroll[12];
                D_pamet[0] =  pom_disp_scroll[13];
                aktualizuj_displej();
              }
            
          }
        else                                                                  // nadmorska vyska se zobrazuje primo
          {
            if (souradnice < 0xFFFFUL)
              {
                pozice_tecky = 0;
                zobraz_cislo(souradnice - 500, 0);            
              }
            else
              {
                zobraz_text(29);                                              // na displeji se zobrazi pomlcky bez scrolovani "-----"
              }
          }
      }  
  }
//----------------------------------------------



//----------------------------------------------
// podle casovych udaju prijatych z modulu GPS pres I2C se nastavi cas v RTC
void GPS_nastav_cas(void)
  {
    byte GMT_rok = pole_GPS_I2C[9];
    byte GMT_mes = pole_GPS_I2C[10];
    byte GMT_den = pole_GPS_I2C[11];
    byte GMT_hod = pole_GPS_I2C[12];
    byte GMT_min = pole_GPS_I2C[13];
    byte GMT_sek = pole_GPS_I2C[14];

    // priprava na autokalibraci casu ------
    // pri kazdem spravnem zadani se vypocita korekce na zaklade rozdilu casu od minuleho serizeni 

    boolean chyba = false;                                                    // bude provadeno testovani na korektni meze vlozenych cisel. Kdyz dojde k prekroceni mezi, nastavi se znacka 'chyba' na true
    boolean bude_korekce = true;                                              // kdyz budou splneny nejake podminky ohledne prechozicho nastaveni casu, provede se vypocet korekce
    long odchylka ;                                                           // o kolik se lisi zadavany cas a aktualni cas v RTC
    unsigned long interval;                                                   // rozdil casu v sekundach od minuleho nastavovani casu
    unsigned long aktualni_cas_RTC;                                           // aktualni cas v RTC v sekundach od 1.1.1970
    
    cas_minuleho_nastaveni = EEPROM_read_long(0);
    aktualni_cas_RTC = rtclock.now();
    interval = aktualni_cas_RTC - cas_minuleho_nastaveni;
    if (interval < 86400 or interval > 63115200)  bude_korekce = false;       // od posledniho nastaveni uplynul mene nez den, nebo vic nez 2 roky - korekce se pocitat nebude
    
    mtt.year     = GMT_rok +30 ;                                              // knihovna je napsana tak, ze se cas pocita od 1.1.1970, GMT se pocita od roku 2000
    mtt.month    = GMT_mes;
    mtt.day      = GMT_den;
    mtt.hour     = GMT_hod;
    mtt.minute   = GMT_min;
    mtt.second   = GMT_sek;
    tt = rtclock.makeTime(mtt);                                               // do RTC se uklada vzdycky GMT (nezavisle na SEC nebo SELC)
    rtclock.setTime(tt);     

    // rezim autokalibrace casu
    odchylka = aktualni_cas_RTC - tt;
    
    if (odchylka > 172800L or odchylka < -172800L )  bude_korekce = false;    // rozdil mezi zadavanym casem a casem v RTC je vic, nez 2 dny. To je CHYBA - korekce se nuluje.
  
    if (bude_korekce == true)                                                 // vypocet korekce (za kolik sekund se ma pripocitat, nebo odecist extra sekunda)
      {
        if (odchylka  > 0) korekce = interval / odchylka;                    // odchylka je kladna, sekundy se budou pricitat
        if (odchylka  < 0) korekce = (0x80000000UL | (interval / -odchylka));// nejvyssi bit je znamenko, odchylka se pak prevadi na kladne cislo
        if (odchylka == 0) korekce = 0x7FFFFFFFUL;                           // cas v RTC je stejny, jako zadany cas - zadna korekce neni potreba (prvni extra sekunda za 68 let) 
      }
    else                                                                      // kdyz je interval mezi serizenimi casu moc kratky, nebo rozdil casu v RTC pro zadanemu casu moc velky ...
      {
        korekce = 0x7FFFFFFFUL;                                                 // ... zadna korekce se pocitat nebude (prvni extra sekunda se pricte az za 68 let) 
      }
  
    cas_minuleho_nastaveni = tt;                                              // nastaveni globalni promenne, aby se nemulo pri kazdem zjistovani casu sahat do EEPROM
  
    if ((korekce & 0x7FFFFFFFUL) < 300)                                         // sekunda se ma pricitat, nebo odecitat driv, nez za 5 minut - to vypada na chybu RTC
      {
        cas_minuleho_nastaveni = 0;
        korekce = 0x7FFFFFFFUL;
        Serial.println(lng204);                                               // Serial.println("Chyba RTC (sekunda se ma korigovat casteji, nez za 5 minut).");
        Serial.println(lng205);                                               // Serial.println("Automaticka kalibrace RTC zrusena.");
      }
  
    EEPROM_write_long(0,cas_minuleho_nastaveni);                              // do EEPROM se ulozi cas posledniho nastaveni (aktualni cas) a vypoctena korekce
    EEPROM_write_long(4,korekce);        
  }
//----------------------------------------------
